@@ -59,6 +59,7 @@ extern "C" {
 #define R_CONS_GREP_WORDS 10
 #define R_CONS_GREP_WORD_SIZE 64
 #define R_CONS_GREP_TOKENS 64
+#define R_CONS_GREP_COUNT 10
 
 R_LIB_VERSION_HEADER(r_cons);
 
@@ -352,20 +353,20 @@ typedef struct r_cons_canvas_t {
 #define RUNE_CURVE_CORNER_BR "╯"
 #define RUNE_CURVE_CORNER_BL "╰"
 #define RUNE_LONG_LINE_HORIZ "―"
-#define UTF_CIRCLE "\u25EF"
-#define UTF_BLOCK "\u2588"
+#define R_UTF8_CIRCLE "\u25EF"
+#define R_UTF8_BLOCK "\u2588"
 
 // Emoji
-#define UTF8_POLICE_CARS_REVOLVING_LIGHT "🚨"
-#define UTF8_WHITE_HEAVY_CHECK_MARK "✅"
-#define UTF8_SEE_NO_EVIL_MONKEY "🙈"
-#define UTF8_SKULL_AND_CROSSBONES "☠"
-#define UTF8_KEYBOARD "⌨"
-#define UTF8_LEFT_POINTING_MAGNIFYING_GLASS "🔍"
-#define UTF8_DOOR "🚪"
+#define R_UTF8_POLICE_CARS_REVOLVING_LIGHT "🚨"
+#define R_UTF8_WHITE_HEAVY_CHECK_MARK "✅"
+#define R_UTF8_SEE_NO_EVIL_MONKEY "🙈"
+#define R_UTF8_SKULL_AND_CROSSBONES "☠"
+#define R_UTF8_KEYBOARD "⌨"
+#define R_UTF8_LEFT_POINTING_MAGNIFYING_GLASS "🔍"
+#define R_UTF8_DOOR "🚪"
 
 // Variation Selectors
-#define UTF8_VS16 "\xef\xb8\x8f"
+#define R_UTF8_VS16 "\xef\xb8\x8f"
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
@@ -380,10 +381,11 @@ typedef enum { COLOR_MODE_DISABLED = 0, COLOR_MODE_16, COLOR_MODE_256, COLOR_MOD
 typedef struct r_cons_context_t {
 	RConsGrep grep;
 	RStack *cons_stack;
-	char *buffer;
+	char *buffer; // TODO: replace with RStrBuf
 	size_t buffer_len;
 	size_t buffer_sz;
-
+	RStrBuf *error; // r_cons_eprintf / r_cons_errstr / r_cons_errmode
+	int errmode;
 	bool breaked;
 	RStack *break_stack;
 	RConsEvent event_interrupt;
@@ -403,6 +405,10 @@ typedef struct r_cons_context_t {
 	int color_mode;
 	RConsPalette cpal;
 	RConsPrintablePalette pal;
+
+	RList *sorted_lines;
+	RList *unsorted_lines;
+	int sorted_column; // -1
 } RConsContext;
 
 #define HUD_BUF_SIZE 512
@@ -428,6 +434,7 @@ typedef struct r_cons_t {
 	int fix_columns;
 	bool break_lines;
 	int noflush;
+	int optimize;
 	bool show_autocomplete_widget;
 	FILE *fdin; // FILE? and then int ??
 	int fdout; // only used in pipe.c :?? remove?
@@ -697,6 +704,7 @@ typedef struct r_cons_canvas_line_style_t {
 	int color;
 	int symbol;
 	int dot_style;
+	const char *ansicolor;
 } RCanvasLineStyle;
 
 // UTF-8 symbols indexes
@@ -784,6 +792,25 @@ R_API int r_cons_win_eprintf(bool vmode, const char *fmt, ...) R_PRINTF_CHECK(2,
 R_API int r_cons_win_vhprintf(DWORD hdl, bool vmode, const char *fmt, va_list ap);
 #endif
 
+#if 0
+
+Flush Print Buffer
+  0     0     0     null
+  0     0     1     quiet
+  0     1     0     echo
+  0     1     1     buffer
+  1     0     1     flush
+
+#endif
+
+enum {
+	R_CONS_ERRMODE_NULL,   // no buffer no print = null
+	R_CONS_ERRMODE_QUIET,  // buffer no print = quiet
+	R_CONS_ERRMODE_ECHO,   // no buffer, print = like eprintf()
+	R_CONS_ERRMODE_BUFFER, // no buffer, print = like eprintf()
+	R_CONS_ERRMODE_FLUSH,  // no buffer, print = like eprintf + log
+};
+
 R_API void r_cons_push(void);
 R_API void r_cons_pop(void);
 R_API RConsContext *r_cons_context_new(R_NULLABLE RConsContext *parent);
@@ -799,6 +826,11 @@ R_API void r_cons_context_break_pop(RConsContext *context, bool sig);
 R_API char *r_cons_editor(const char *file, const char *str);
 R_API void r_cons_reset(void);
 R_API void r_cons_reset_colors(void);
+R_API char *r_cons_errstr(void);
+R_API void r_cons_errmode(int mode);
+R_API void r_cons_errmodes(const char *mode);
+R_API int r_cons_eprintf(const char *format, ...);
+R_API void r_cons_eflush(void);
 R_API void r_cons_print_clear(void);
 R_API void r_cons_echo(const char *msg);
 R_API void r_cons_zero(void);
@@ -812,6 +844,7 @@ R_API void r_cons_stdout_open(const char *file, int append);
 R_API int  r_cons_stdout_set_fd(int fd);
 R_API void r_cons_gotoxy(int x, int y);
 R_API int r_cons_get_cur_line(void);
+R_API void r_cons_line(int x, int y, int x2, int y2, int ch);
 R_API void r_cons_show_cursor(int cursor);
 R_API char *r_cons_swap_ground(const char *col);
 R_API bool r_cons_drop(int n);
@@ -832,7 +865,7 @@ R_API void r_cons_println(const char* str);
 
 R_API void r_cons_strcat_justify(const char *str, int j, char c);
 R_API void r_cons_printat(const char *str, int x, char y);
-R_API int r_cons_memcat(const char *str, int len);
+R_API int r_cons_write(const char *str, int len);
 R_API void r_cons_newline(void);
 R_API void r_cons_filter(void);
 R_API void r_cons_flush(void);
@@ -924,6 +957,23 @@ R_API void r_cons_bind(RConsBind *bind);
 R_API const char* r_cons_get_rune(const ut8 ch);
 #endif
 
+/* pixel.c */
+typedef struct {
+	int w;
+	int h;
+	ut8 *buf;
+	size_t buf_size;
+} RConsPixel;
+
+R_API RConsPixel *r_cons_pixel_new(int w, int h);
+R_API void r_cons_pixel_free(RConsPixel *p);
+R_API void r_cons_pixel_flush(RConsPixel *p, int sx, int sy);
+R_API char *r_cons_pixel_drain(RConsPixel *p);
+R_API void r_cons_pixel_set(RConsPixel *p, int x, int y, int v);
+R_API void r_cons_pixel_sets(RConsPixel *p, int x, int y, const char *s);
+R_API void r_cons_pixel_fill(RConsPixel *p, int _x, int _y, int w, int h, int v);
+R_API char *r_cons_pixel_tostring(RConsPixel *p);
+
 /* r_line */
 #define R_LINE_BUFSIZE 4096
 #define R_LINE_HISTSIZE 256
@@ -947,10 +997,12 @@ typedef struct r_selection_widget_t {
 
 typedef struct r_line_hist_t {
 	char **data;
+	char *match;
 	int size;
 	int index;
 	int top;
 	int autosave;
+	bool do_setup_match;
 } RLineHistory;
 
 typedef struct r_line_buffer_t {
